function createPlayer()
{
	%obj = new AIPlayer()
	{
		datablock = playerStandardArmor;
	};

	if ( !isObject( %obj ) )
	{
		return -1;
	}

	missionCleanup.add( %obj );
	%obj.playThread( 1, "armReadyBoth" );

	%obj.setDecalName( "Mod-Suit" );
	%obj.setNodeColor( "ALL", "1 0.878 0.611 1" );

	%obj.setNodeColor( "chest", "0.2 0.2 0.2 1" );
	%obj.setNodeColor( "pants", "0 0.141 0.333 1" );

	%obj.setNodeColor( "larm", "0.5 0.5 0.5 1" );
	%obj.setNodeColor( "rarm", "0.5 0.5 0.5 1" );

	%obj.setNodeColor( "lshoe", "0 0.141 0.333 1" );
	%obj.setNodeColor( "rshoe", "0 0.141 0.333 1" );

	return %obj;
}

function createVehicle( %db )
{
	if ( !isObject( %db ) )
	{
		return -1;
	}

	if ( %db.getClassName() !$= "WheeledVehicleData" )
	{
		return -1;
	}

	%obj = new wheeledVehicle()
	{
		datablock = %db;
	};

	if ( !isObject( %obj ) )
	{
		return -1;
	}

	missionCleanup.add( %obj );
	return %obj;
}

function getPath( %count, %prefix )
{
	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%path = %path @ ( %i ? " " : "" ) @ "_" @ %prefix @ %i;
	}

	return %path;
}

// Driving

function getNearestRoadPiece( %pos )
{
	%pos = setWord( %pos, 2, 0 );
	%num = roadGroup.getCount();

	for ( %idx = 0 ; %idx < %num ; %idx++ )
	{
		%test = roadGroup.getObject( %idx );
		%dist = vectorDist( %pos, setWord( %test.position, 2, 0 ) );

		if ( %dist < %best || !strLen( %best ) )
		{
			%best = %dist;
			%road = %test;
		}
	}

	return strLen( %road ) ? %road : -1;
}

function wheeledVehicle::getPassengerCount( %this )
{
	%count = %this.getDataBlock().numMountPoints;
	%total = -1;

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		if ( isObject( %this.getMountedObject( %i ) ) )
		{
			%total += 1;
		}
	}

	return mClamp( %total, 0, %count - 1 );
}

function AIPlayer::driveRespondNearby( %this, %vehicle, %speed, %current )
{
	%point = %vehicle.getPosition();
	%range = 64;
	%masks = $TypeMasks::VehicleObjectType | $TypeMasks::PlayerObjectType;

	initContainerRadiusSearch( %point, %range, %masks );

	while ( isObject( %obj = containerSearchNext() ) )
	{
		if ( %obj $= %vehicle )
		{
			continue;
		}

		if ( %obj.getType() & $TypeMasks::PlayerObjectType )
		{
			%player = true;

			if ( %obj.isMounted() )
			{
				continue;
			}
		}
		else
		{
			%player = false;
		}

		%diff = getAngleDifference( %vehicle, %obj );

		if ( mAbs( %diff ) >= 0.25 )
		{
			continue;
		}

		%dist = vectorDist( %vehicle.position, %obj.position );

		if ( %dist < %minDist || !strLen( %minDist ) )
		{
			%minDist = %dist;
			%isPlayer = %player;
		}
	}

	if ( strLen( %minDist ) && %minDist <= 32 )
	{
		%factor = %minDist / 32;
		%factor = %factor * 1.65 - 0.65;

		if ( %factor < 0 && %isPlayer )
		{
			%factor = 0;
		}

		%speed *= mClampF( %factor, -1, 1 );

		if ( %speed < 0 )
		{
			%speed *= 4;
		}

		// %speed *= mClampF( ( %minDist - 16 ) / ( 32 - 16 ), 0, 1 );
	}

	if ( %speed >= 16 )
	{
		%next = %this.getNextTurn( %vehicle );
		// %prev = %this.getPrevTurn( %vehicle );
		%prev = -1;

		if ( %next == -1 )
		{
			%next = 256;
		}

		if ( %prev == -1 )
		{
			%prev = 256;
		}

		%dist = %next > %prev ? %prev : %next;
		%start = 16 + mClampF( %current / 32, 0, 1 ) * 16;

		if ( %dist >= 0 && %dist <= %start )
		{
			%speed *= mClampF( %dist / %start, 0.35, 1 );

			if ( %speed < 16 )
			{
				%speed = 16;
			}
		}
	}

	// %vehicle.setShapeName( "Target Speed: " @ mFloatLength( %speed, 0 ) SPC "km/h" );
	// %vehicle.setShapeName( %prev );
	return %speed;
}

function AIPlayer::getNextTurn( %this, %vehicle )
{
	%start = %this.index;
	%first = true;

	%distance = 0;

	for ( %i = %this.index ; true ; %i = ( %i + 1 ) % %this.count )
	{
		%node = getWord( %this.path, %i );
		%prev = getWord( %this.path, ( %i - 1 ) % %this.count );

		if ( %i == %this.index )
		{
			if ( %first )
			{
				%distance += vectorDist( %vehicle.position, %node.position );

				%angle = vectorSub( %node.position, %prev.position );
				%angle = vectorNormalize( %angle );
			}
			else
			{
				break;
			}
		}
		else
		{
			%distance += vectorDist( %node.position, %prev.position );

			%test = vectorSub( %node.position, %prev.position );
			%test = vectorNormalize( %test );

			if ( %test !$= %angle )
			{
				%found = true;
				break;
			}
		}

		%first = false;
	}

	%distance -= 16;

	if ( %found )
	{
		return %distance;
	}

	return -1;

	// %factor = %turnDist / 24;
	// %speed *= mClampF( %factor, 0.2, 1 );
}

function AIPlayer::getPrevTurn( %this, %vehicle )
{
	%start = %this.index;
	%first = true;

	%distance = 0;

	for ( %i = %this.index ; true ; %i = ( %i - 1 ) % %this.count )
	{
		%node = getWord( %this.path, %i );
		%next = getWord( %this.path, ( %i + 1 ) % %this.count );

		if ( %i == %this.index )
		{
			if ( %first )
			{
				%distance += vectorDist( %vehicle.position, %node.position );

				%angle = vectorSub( %node.position, %next.position );
				%angle = vectorNormalize( %angle );
			}
			else
			{
				break;
			}
		}
		else
		{
			%distance += vectorDist( %node.position, %next.position );

			%test = vectorSub( %node.position, %next.position );
			%test = vectorNormalize( %test );

			if ( %test !$= %angle )
			{
				%found = true;
				break;
			}
		}

		%first = false;
	}

	%distance -= 16;

	if ( %found )
	{
		return %distance;
	}

	return -1;
}