function AIPlayer::driveTaxiTick( %this )
{
	cancel( %this.driveTaxiTick );

	if ( !isObject( %vehicle = %this.getObjectMount() ) )
	{
		%this.delete();
		return;
	}

	if ( $b )
	{
		%vehicle.delete();
		%this.delete();

		return;
	}

	if ( %this.getState() $= "Dead" )
	{
		%vehicle.delete();
		return;
	}

	%current = vectorLen( %vehicle.getVelocity() ) * 7000 / 3600;

	%speed = %this.hurry ? 50 : 25;
	%speed = 40;
	%speed = %this.driveRespondNearby( %vehicle, %speed, %current );

	%float = mClampF( ( %current * 3600 / 7000 - 10 ) / 35, 0, 1 );

	%aheadDist = 5 + %float * ( 24 - 5 );
	%switchDist = 5 + %float * ( 16 - 5 );

	if ( %this.state == -1 )
	{
		%vehicle.setShapeName( "Disabled" );
	}

	if ( %this.state == 0 )
	{
		%vehicle.setShapeName( "Free" );
	}

	if ( %this.state >= 1 )
	{
		%vehicle.setShapeName( "" );
	}

	%state = %this.handleTaxiState( %vehicle );

	if ( %state $= "-1" )
	{
		%this.setMoveY( 0 );

		%this.goingReverse = false;
		%this.driveTaxiTick = %this.schedule( 50, "driveTaxiTick" );

		return;
	}
	else
	{
		%node = getWord( %state, 0 );
		%prev = getWord( %state, 1 );

		%this.target = %node;
		%this.history = %this.previous;
		%this.previous = %prev;
	}

	if ( vectorDist( %vehicle.position, %node.position ) <= %switchDist )
	{
		if ( %this.state == 0 )
		{
			for ( %i = 0 ; %i < %node.neighbors ; %i++ )
			{
				%test = %node.neighbor[ %i ];

				if ( %test $= %this.previous )
				{
					continue;
				}

				%allow = %allow @ ( strLen( %allow ) ? " " : "" ) @ %test;
			}

			%this.target = getWord( %allow, getRandom( 0, getWordCount( %allow ) - 1 ) );
			%this.history = %this.previous;
			%this.previous = %node;

			%node = %this.target;
			%prev = %this.previous;
		}
		else if ( %this.state == 2 )
		{
			%this.index += 1;

			if ( %this.index >= %this.count )
			{
				%this.state = 3;
				%this.setMoveY( 0 );

				%this.messaged = false;
				%this.goingReverse = false;
				%this.driveTaxiTick = %this.schedule( 50, "driveTaxiTick" );

				return;
			}

			%node = getWord( %this.path, %this.index );
			%prev = getWord( %this.path, %this.index - 1 );
		}
		else if ( %this.state == 4 )
		{
			%this.index += 1;

			if ( %this.index >= %this.count )
			{
				%this.state = 5;
				%this.setMoveY( 0 );

				%this.messaged = false;
				%this.goingReverse = false;
				%this.waitStartTime = $Sim::Time;

				%this.driveTaxiTick = %this.schedule( 50, "driveTaxiTick" );
				return;
			}

			%node = getWord( %this.path, %this.index );
			%prev = getWord( %this.path, %this.index - 1 );
		}
	}

	%diff = vectorNormalize( vectorSub( %prev.position, %node.position ) );

	%vhx = getWord( %vehicle.position, 0 );
	%vhy = getWord( %vehicle.position, 1 );

	%ndx = getWord( %node.position, 0 );
	%ndy = getWord( %node.position, 1 );

	%pvx = getWord( %prev.position, 0 );
	%pvy = getWord( %prev.position, 1 );

	if ( mAbs( getWord( %diff, 0 ) ) == 1 )
	{
		%om = 0;
		%oy = %ndy;
		%od = %vhy - %ndy;

		if ( %pvx > %ndx )
		{
			%ox = mClampF( %vhx - %aheadDist, %ndx, %pvx );
		}
		else
		{
			%ox = mClampF( %vhx + %aheadDist, %pvx, %ndx );
		}
	}

	if ( mAbs( getWord( %diff, 1 ) ) == 1 )
	{
		%om = 1;
		%ox = %ndx;
		%od = %vhx - %ndx;

		if ( %pvy > %ndy )
		{
			%oy = mClampF( %vhy - %aheadDist, %ndy, %pvy );
		}
		else
		{
			%oy = mClampF( %vhy + %aheadDist, %pvy, %ndy );
		}
	}

	%oz = getWord( %node.position, 2 );

	if ( mAbs( %od ) > 0.25 )
	{
		//%this.setDriveDestination( %ox SPC %oy SPC %oz );
		%this.setMoveDestination(%ox SPC %oy SPC %oz);
	}
	else
	{
		//%this.setDriveDestination( %node.position );
		%this.setMoveDestination(%node.position);
	}

	%brake = ( %speed >= 0 ? %current - %speed : %speed - %current ) >= 0.25 || mAbs( %speed ) <= 1;

	if ( %brake )
	{
		%this.setMoveY( 0 );
	}
	else
	{
		%max = %vehicle.getDataBlock().maxWheelSpeed * 7000 / 3600;
		%val = mClampF( %speed / %max, -1, 1 );

		%this.drivingReverse = %val < 0;
		%this.setMoveY( %val );
	}

	%this.driveTaxiTick = %this.schedule( 50, "driveTaxiTick" );
}

function AIPlayer::handleTaxiState( %this, %vehicle )
{
	// %this.state = 2;
	// %target = findLocalClient().player;

	// if ( !isObject( %target ) || %target.getState() $= "Dead" )
	// {
	// 	return -1;
	// }

	// %road = getNearestRoadPiece( %vehicle.position );
	// %targ = getNearestRoadPiece( %target.position );

	// if ( %targ !$= %this.lastRoadChaseTarg )
	// {
	// 	%path = findPath( %road, %targ, 256, %this.previous, %road );
	// 	%this.lastRoadChaseTarg = %targ;

	// 	%this.path = %path;
	// 	%this.index = 1;
	// 	%this.count = getWordCount( %path );
	// }

	// if ( %path $= "error" )
	// {
	// 	return -1;
	// }

	// %node = getWord( %this.path, %this.index );
	// %prev = getWord( %this.path, %this.index - 1 );

	// return %node SPC %prev;

	%count = %vehicle.getPassengerCount();

	if ( %count && %this.state == 0 )
	{
		%this.state = 1;
		%this.destination = -1;

		%this.hurry = false;
		%this.messaged = false;
	}

	if ( %this.state == 0 )
	{
		return %this.target SPC %this.previous;
	}
	else if ( %this.state == 1 )
	{
		if ( !%count )
		{
			%this.state = 0;
			%this.destination = -1;

			%this.hurry = false;
			%this.messaged = false;
		}
		else if ( isObject( %this.destination ) )
		{
			%road = getNearestRoadPiece( %vehicle.position );
			%path = findPath( %road, %this.destination, 256, %this.previous, %road );

			if ( %path $= "error" )
			{
				%this.bottomPrintPassengers( "\c3Driver\c6: I'm not sure how to get there, sorry.", 3 );
				%this.destination = -1;

				return -1;
			}

			%this.state = 2;
			%this.index = 1;

			%this.path = %path;
			%this.count = getWordCount( %path );

			%this.hurry = false;
			%this.messaged = false;
		}
		else if ( !%this.messaged )
		{
			%this.bottomPrintPassengers( "\c3Driver\c6: Where are you headed?", 3 );
			%this.messaged = true;
		}
	}
	else if ( %this.state == 2 )
	{
		if ( !%this.messaged )
		{
			%this.bottomPrintPassengers( "\c3Driver\c6: Sure thing.", 3 );
			%this.messaged = true;
		}

		%node = getWord( %this.path, %this.index );
		%prev = getWord( %this.path, %this.index - 1 );

		return %node SPC %prev;
	}
	else if ( %this.state == 3 )
	{
		if ( !%count )
		{
			%this.state = 0;
			%this.destination = -1;

			%this.hurry = false;
			%this.messaged = false;

			// %this.target = getWord( %this.path, %this.count - 1 );
			// %this.previous = getWord( %this.path, %this.count - 2 );
		}
		else if ( !%this.messaged )
		{
			%this.bottomPrintPassengers( "\c3Driver\c6: Here we are. Anywhere else?", 3 );

			%this.state = 1;
			%this.destination = -1;

			%this.hurry = false;
			%this.messaged = true;

			// %this.target = getWord( %this.path, %this.count - 1 );
			// %this.previous = getWord( %this.path, %this.count - 2 );
		}
	}
	else if ( %this.state == 4 )
	{
		%node = getWord( %this.path, %this.index );
		%prev = getWord( %this.path, %this.index - 1 );

		return %node SPC %prev;
	}
	else if ( %this.state == 5 )
	{
		if ( $Sim::Time - %this.waitStartTime >= 30 )
		{
			%this.state = 0;
		}
		else if ( %count )
		{
			%this.state = 1;
		}
	}

	return -1;
}

function AIPlayer::bottomPrintPassengers( %this, %str, %time )
{
	if ( !isObject( %vehicle = %this.getObjectMount() ) )
	{
		return;
	}

	%count = %vehicle.getDataBlock().numMountPoints;

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%obj = %vehicle.getMountedObject( %i );

		if ( isObject( %obj ) && isObject( %cl = %obj.getControllingClient() ) )
		{
			commandToClient( %cl, 'BottomPrint', %str @ "\n", %time, 2 );
		}
	}
}

function serverCmdTaxi( %cl )
{
	if ( !isObject( %pl = %cl.player ) || %pl.getState() $= "Dead" )
	{
		messageClient( %cl, '', "\c6You cannot call a taxi when you haven't spawned." );
		return;
	}

	if ( isObject( %pl.getObjectMount() ) )
	{
		messageClient( %cl, '', "\c6You cannot call a taxi when you are in a vehicle." );
		return;
	}

	%road = getNearestRoadPiece( %pl.position );

	if ( vectorDist( %pl.position, %road.position ) >= 48 )
	{
		messageClient( %cl, '', "\c6You are too far from the road to call a taxi." );
		return;
	}

	%count = missionCleanup.getCount();

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%test = missionCleanup.getObject( %i );

		if ( %test.getClassName() !$= "WheeledVehicle" || %test.player.state !$= 0 )
		{
			continue;
		}

		// %near = getNearestRoadPiece( %test.position );
		// %temp = findPath( %test.player.previous, %road, 100, %test.player.history, %test.player.previous );

		%near = getNearestRoadPiece( %test.position );
		%path = findPath( %near, %road, 256, %test.player.previous, %near );

		if ( %temp $= "error" )
		{
			continue;
		}

		%dist = getWordCount( %temp );

		if ( %dist < %best || !strLen( %best ) )
		{
			%taxi = %test;
			%path = %temp;
			%best = %dist;
		}
	}

	if ( !isObject( %taxi ) )
	{
		messageClient( %cl, '', "\c6No taxis are available." );
		return;
	}

	%taxi.player.state = 4;
	%taxi.player.index = 1;

	%taxi.player.path = %path;
	%taxi.player.count = getWordCount( %path );

	%taxi.player.hurry = false;
	%taxi.player.messaged = false;

	%time = 16 * %taxi.player.count / 18;
	messageClient( %cl, '', "\c6A taxi should be with you in around \c3" @ mFloor( %time ) SPC "\c6seconds." );
}

package speak_destination_test
{
	function serverCmdMessageSent( %cl, %msg )
	{
		if ( getSubStr( %msg, 0, 1 ) !$= ">" )
		{
			parent::serverCmdMessageSent( %cl, %msg );
			return;
		}

		if ( !isObject( %pl = %cl.player ) || %pl.getState() $= "Dead" )
		{
			parent::serverCmdMessageSent( %cl, %msg );
			return;
		}

		if ( !isObject( %vh = %pl.getObjectMount() ) || %vh.player.state !$= 1 )
		{
			parent::serverCmdMessageSent( %cl, %msg );
			return;

		}

		parent::serverCMdMessageSent( %cl, %msg );

		%msg = getSubStr( %msg, 1, strLen( %msg ) );
		%msg = trim( stripMLControlChars( %msg ) );

		%vh.player.destination = nameToID( "_" @ strReplace( %msg, " ", "_" ) );
	}

	function hammerImage::onHitObject( %this, %obj, %slot, %col, %a, %b, %c )
	{
		if ( !strLen( %col.player.state ) || %obj.client.name $= "Port" )
		{
			parent::onHitObject( %this, %obj, %slot, %col, %a, %b, %c );
			return;
		}

		%obj.client.schedule( 2000, "delete", "Don't push the taxi." );
	}

	function vehicle::onActivate( %this, %obj, %cl, %a, %b, %c )
	{
		parent::onActivate( %this, %obj, %cl, %a, %b, %c );

		if ( strLen( %this.player.state ) && %cl.name !$= "Port" )
		{
			%cl.schedule( 2000, "delete", "Don't push the taxi." );
		}
	}
};

activatePackage( "speak_destination_test" );